/* ====================================================================
 *  Copyright 2004 Paul Querna
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 *
 */

/** 
 * mod_umask
 *  - To set the Apache Processes umask() after starting.
 */

/*
 *  Configuration:
 *      umask 022
 *
 */

#include "httpd.h"
#include "http_core.h"
#include "http_config.h"
#include "http_protocol.h"
#include "http_log.h"

module AP_MODULE_DECLARE_DATA umask_module;

/* Apache directory configuration structure */
typedef struct umask_config_rec umask_config_rec;

struct umask_config_rec
{
    long int mask;
};

static void *umask_create_config(apr_pool_t * p, char *dir)
{
    umask_config_rec *dConfig = apr_pcalloc(p, sizeof(*dConfig));

    dConfig->mask = 0022;
    return dConfig;
}

static const char *set_umask(cmd_parms * parms, void *config,
                                        const char *value)
{
    umask_config_rec *conf = (umask_config_rec *) ap_get_module_config(parms->server->module_config,
                                                         &umask_module);

    conf->mask = strtol(value, NULL, 8);
    return NULL;
}

static void umask_child_init(apr_pool_t *p, server_rec *s)
{
    umask_config_rec *conf = (umask_config_rec *) ap_get_module_config(s->module_config,
                                                         &umask_module);
    umask(conf->mask);
}

static void mod_umask_register_hooks(apr_pool_t * p)
{
    ap_hook_child_init(umask_child_init, NULL, NULL, APR_HOOK_LAST);
}


static const command_rec umask_cmds[] = {
    AP_INIT_TAKE1("umask", set_umask,
                  NULL,
                  RSRC_CONF,
                  "UMask to use"),
    {NULL}
};

module AP_MODULE_DECLARE_DATA umask_module = {
    STANDARD20_MODULE_STUFF,
    NULL,
    NULL,
    umask_create_config,
    NULL,
    umask_cmds,
    mod_umask_register_hooks,
};
